/* sw_tk.c -- this file determines whether speech is available on the
 *            scout and executes the tk command on behalf of the client. 
 *       -rak 24jun1999
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "sw_tk.h"

const char *speech_prog[] = {"festival", "--pipe", NULL};
const char cmd_begin[] = "(SayText \"";
const char cmd_end[]   = "\")\n";

/* function prototypes */
void setup_child(void);
int start_speech(void);

enum {
  TK_UNINIT=0,
  TK_PRESENT,
  TK_ABSENT
};


static int tk_avail = TK_UNINIT;
int pipefd[2]; 		/* in linux, fd[0] receives while fd[1] sends */

/* setup_child(): only called by child process.  exec the speech synthesizer.
 *      any return indicates an error 
 */
void setup_child()
{
  dup2(pipefd[0], 0);	/* copy pipe-out to stdin of child */
  close(pipefd[0]);
  close(pipefd[1]);
  execvp(speech_prog[0], (char**)speech_prog);
  perror("execvp");
}


/* find out if software speech is available on the present system.
 * returns 1 if it is available and 0 otherwise.
 */
int start_speech()
{
  /* if speech is already running, do nothing */
  if (tk_avail == TK_PRESENT)
    return 1;

  tk_avail = TK_PRESENT;	/* assume that it works until proven otherwise*/

  if (pipe(pipefd) < 0)
    return 0;

  if (fork() == 0) {
    /* the child thread */
    setup_child();
    /* if we get here, the speech program could not be started */
    printf("unable to start child\n");
    exit(1);	/* exit the child process */
  }
  else {
    /* the parent thread */
    close(pipefd[0]);
  }

  return tk_avail == TK_PRESENT;	/* return true if speech is present */
}


/* sw_tk(): attempt to start the speech software and send the requested
       string to it */
int sw_tk(struct tk_msg *msg)
{
  unsigned short len;
  int ret;
  if (!start_speech())
    return 0;

  len = msg->len0 + (msg->len1 << 8);
  ret = write(pipefd[1], cmd_begin, strlen(cmd_begin));
  ret = write(pipefd[1], msg->data, strlen(msg->data));
  ret = write(pipefd[1], cmd_end, strlen(cmd_end));
/*
  close(pipefd[1]);
  wait(&status);
  tk_avail = TK_UNINIT;
*/
  return 1;
}


int sw_tk_str(char *str)
{
  /* allocate space for the structure, string and string terminator */
  /* struct tk_msg *msg = alloca(sizeof(struct tk_msg) + strlen(str) + 1);*/
  char buf[1024];
  struct tk_msg *msg = (struct tk_msg*)buf;
  int len = strlen(str);
  strcpy(msg->data, str);
  msg->len0 = len & 0xff;
  msg->len1 = (len >> 8) & 0xff;
  return sw_tk(msg);
}
