/* examples/armtest.c
 *
 * example program for using the Nomad200 Lift Mechanism
 *
 */

#include <stdio.h>

#include "Nclient.h"


/* flag -- do we want to zero the gripper as well as the lift? */
#undef NO_GRIP



int main(void)
{
  long res, time;

  /* connect to robot */
  connect_robot(1);

  /* check to see if arm is oriented before moving it */
#ifdef NO_GRIP
  arm_zr(ZR_NO_N_GRIPPER | ZR_CHECK);
#else
  arm_zr(ZR_CHECK);
#endif

  /* set maximum torques to 100% */
  arm_mv(MV_MT, 100, MV_MT, 100);

  /* set accelerations */
  arm_mv(MV_AC, 100, MV_AC, 100);

  /* move lift axis to home position 
     move gripper out to maximum positive position */
  arm_mv(MV_PA, 0, MV_VM, 30);

  /* wait for lift and gripper to finish */
  arm_ws(1, 1, 10, NULL);

  /* set maximum torque for gripper to 40% */
  arm_mv(0, 0, MV_MT, 40);

  /* close gripper */
  arm_mv(0, 0, MV_VM, -10);

  /* wait for gripper to finish or error */
  res=arm_ws(0, 1, 10, &time);

#ifndef NO_GRIP
  if (res == 0)
  {
    printf("Gripper closed successfully with %ld milliseconds left.\n", time);
  }
  else
  {
    printf("Gripper unable to close, gripped object.\n");
  }
#endif

  /* move lift axis to top of travel */
  arm_mv(MV_VM, 50, 0, 0);

  /* wait for lift to finish */
  arm_ws(1, 0, 10, NULL);

  return 0;
}
