/* examples/sonar_bounce_n200.c
 *
 * example program demonstrating simple motion control and sensing on
 * Nomad200 and Nomad150 mobile robots.
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "Nclient.h"



static void turn_front_sonar_on(void)
{
  /* remainder are irrelevant */
  int sn_order[16] = {0, 2, 15, 1, 14, 3, 13, 4, 12, 5, 11, 6, 10, 7, 9, 8};
  
  /* turn on 5 front sonars and set them to fire in certain order */
  conf_sn(15, sn_order);
  
  conf_tm(5); 
}



/* monitor the active sonar as specified in parameter "order"
 * returns the sonar which detects the closest object and is less
 * than min_dist */
static short shortest_front_sonar(long *state, short min_dist)
{
  short i, shortest, shortest_sonar;
  
  shortest = 255;
  shortest_sonar = 0;
  
  i = 17;
  do
  {
    if (state[i] < shortest)
    {
      shortest = (short)state[i];
      shortest_sonar = i - 16;
    }
    printf("%d: %d ", i - 17, (int)state[i]);
    i++;
    if (i == 20)
    {
      i = 31;
    }
  } while (i < 33);
  
  if (shortest > min_dist)
  {
    shortest_sonar = 0;
  }
  
  printf("\n");
  return(shortest_sonar);
}



static void wander(short track_dist)
{
  short del_angle;
  short s_v = 0;
  short vmax = 100;
  short shortest_front;
  static short cwturncounter;
  static short ccwturncounter;
  static short lastdir;

  shortest_front = shortest_front_sonar(State, track_dist);
  
  if (shortest_front == 0)
  {
    vm(vmax, 0, 0);
    ccwturncounter =cwturncounter = lastdir = 0;
    return;
  }

  if (shortest_front < 5)
  {
    /* Check for getting stuck */
    if ((cwturncounter > 5) && (ccwturncounter > 5))
    {
      shortest_front = (shortest_front + 3);
    }
    else
    {
      shortest_front = -(shortest_front + 3);
    }
    if (lastdir == 0)
    {
      cwturncounter += 1;
    }
    lastdir = 1;
  }
  if (shortest_front > 12)
  {
    shortest_front = shortest_front - 12;
    if (lastdir == 1)
    {
      ccwturncounter += 1;
    }
    lastdir = 0;
  }
  del_angle = shortest_front*225;
  s_v = del_angle/4;
  vm(0, s_v, s_v);
}



int main(void)
{
  /* connect to the robot (int this example, directly to a Nomad200 robot
   * using Ndirect.o) */
  connect_robot(1, MODEL_N200, "/dev/ttyS0", 38400);

  /* zero the robot */
  zr();

  /* turn on the front sonars */
  turn_front_sonar_on();
  
  /* the loop */
  gs();

  while(1)
  {
    wander(20);
  }
  
  return 0;
}


